/**
 * @(#)examiner2.cpp 2003/07/08
 *
 * @author Ice
 * @version 1.00
 *
 * ICE Team free software group
 *
 * This file is part of TSID2.
 * See README for copyright notice.
 *
 * note: some comments are in the style of Javadoc of Java language!
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
 *  02111-1307  USA.
 */

//#include "osdepend.h"
//#ifdef HAVE_UNIX
//  #include "tsidex.h"
//#else
  #include "tsidexaminer2.h"
//#endif

#include <stdio.h>
#include <stdlib.h>
#include <iostream.h>
#include <fstream.h>
#include <iomanip.h>
#include <string.h>

// paramenter field
const int P_NOTHING   = 0x0000;  // nothing passed
const int P_UNSORTED  = 0x0001;  // unsorted list
const int P_SORTED    = 0x0002;  // sorted list
const int P_AUTHOR    = 0x0004;  // author list
const int P_TUNE      = 0x0008;  // single tune list
const int P_SONG      = 0x0010;  // songs (files) list
const int P_TIME      = 0x0020;  // listen time form
const int P_NUMBER    = 0x0040;  // number of listen form
const int P_PONDERED  = 0x0080;  // pondered form (listen/number)
const int P_HELP      = 0x0100;  // help of commands
const int P_ERROR     = 0x0200;  // error in parsing parameters
const int P_PLAYLIST  = 0x0400;  // playlist generation
const int P_PATH      = 0x0800;  // user specified path
const int P_NOINDEX   = 0x1000;  // no use the index
///const int P_IPATH     = 0x2000;  // user specified path for index file

char *DOC="DOCUMENTS";
char *DBN="Songlengths.txt";

char *ULOST   = "ulost.spl";    // Unsorted List Of Single Tunes
char *LTSLOST = "ltslost.spl";  // Listen Time Sorted List Of Single Tunes
char *NLSLOST = "nlslost.spl";  // Number Listen Sorted List Of Single Tunes
char *PSLOST  = "pslost.spl";   // Pondered Sorted List Of Single Tunes
char *ULOS    = "ulos.spl";     // Unsorted list of songs
char *LTSLOS  = "ltslos.spl";   // Listen Time Sorted List Of Songs
char *NLSLOS  = "nlslos.spl";   // Number Listen Sorted List Of Songs
char *PSLOS   = "pslos.spl";    // Pondered Sorted List Of Songs

char *env=getenv("HVSC_DIR");   // enviroment HVSC directory

char *path;                     // path on where search for TSID2 tree
char *home;                     // home directory
///char *ipath;                    // path to index.ini file
 
//#ifdef HAVE_UNIX
//  TSIDEx examiner;
//#else
  // no pondered option supported for windows user
  TSIDExaminer2 *examiner;
//#endif


#ifdef HAVE_UNIX
#  define TSID2DIR ".tsid2"
#else
#  define TSID2DIR "tsid2"
#  define PATH_SEP '\\'
#endif

/**
 * Get home directory of user
 * The variables path is chaged
 * Note: this code is give by Simon White
 *
 * @return true if operation is ok
 */
bool getHomeDir() {
  char *lpath = (char *) getenv ("HOME");
  //lastError=NO_TSID_ERROR;

  if (!lpath)
      lpath = (char *) getenv ("windir");
  if (!lpath) {
      cerr << "No home directory of uesr found\n";
      return false;
  }

  home=new char[strlen(lpath)+1];
  if (home==NULL) {
      cerr << "No more memory\n";
      return false;
  }
  strcpy(home, lpath);
  return true;
}

/**
 * Convert platform specific directory seperator to ansi
 *
 * @param filename the filename to made with '/'
 */
void convertDirSeperator(char *filename) {
#ifndef HAVE_UNIX
  char *p = filename;
  while (*p != '\0') {
    if (*p == PATH_SEP) *p = '/';
    p++;
  }
#endif
}

/**
 * Get parameters summary as an integer value
 *
 * @param argc the numbr of passed string
 * @param argv the strings of parameters
 * @return an integer that summary all the options available
 */
int getParameter(int argc, char** argv) {
  int value=P_NOTHING;                       // returned value;
  int tmp;
  const char *buf;
  int b;

  // parse command line arguments
  int a=1;
  while ((a<argc) && (argv[a] != NULL))	{
    if (argv[a][0]=='-') {
      b=1;
      while (argv[a][b]!='\0') {
        switch (argv[a][b]) {
          case 'u':                          // unsorted list
            value|=P_UNSORTED;
            break;
          case 's':                          // sorted list
            value|=P_SORTED;
            break;
          case 'a':                          // author list
            value|=P_AUTHOR;
            break;
          case 't':                          // single tune list
            value|=P_TUNE;
            break;
          case 'f':                          // songs (file) list
            value|=P_SONG;
            break;
          case 'l':                          // listen time form
            value|=P_TIME;
            break;
          case 'n':                          // number of listen form
            value|=P_NUMBER;
            break;
          case 'p':                          // pondered form
            value|=P_PONDERED;
            break;
          case 'h':                          // help
            value|=P_HELP;
            break;
          case 'g':                          // playlist generation
            value|=P_PLAYLIST;
            break;
          case 'b':                          // base path
            value|=P_PATH;
            b++;
            tmp=b;
            while (argv[a][b]!='\0') {
              b++;
            }  
            // now the path string is from tmp to b position
            int length;
            path=new char[(length=(b-tmp))+1];
            buf=&argv[a][tmp];            
            strncpy(path, buf, length);
            b--;                             // point pointer back before 0 position
            break;    
          case 'x':                          // no use the index      
            value|=P_NOINDEX;
            break;                      
          ///case 'i':                          // base path for index
          ///  value|=P_IPATH;
          ///  b++;
          ///  tmp=b;
          ///  while (argv[a][b]!='\0') {
          ///    b++;
          ///  }  
            // now the path string is from tmp to b position
          ///  ipath=new char[(length=(b-tmp))+1];
          ///  buf=&argv[a][tmp];            
          ///  strncpy(ipath, buf, length);
          ///  b--;                             // point pointer back before 0 position
          ///  break;                     
          default:
            return P_ERROR;
        }
        b++;                                 // next char
      }
    } else {
        // option must be specify with -
        return P_ERROR;
      }

    a++;                                     // next parameter
  }
  return value;
}

/**
 * Add a entry in the playlist.
 *
 * @param rec the rec of information
 * @param out the output (file) stream 
 */
void addPlaylistEntry(TSIDExaminer2::TRec rec, ostream *out) {
  char buf[0x80];

  // path and name of file
  char *pName;

  pName=new char[strlen(rec.name)+strlen(rec.path)+strlen(env)+1];

  if (pName==NULL) {
      cerr << "Error allocating memory\n";
      return;
  }

  sprintf(pName, "%s%s%s", env, rec.path, rec.name);

  // open the file
  ifstream file(pName, ios::in);

  if (!file) {
    cerr << "no HVSC file found: missing file: " << rec.name << endl
         << "There is something wrong in your"
         << " collection or in the TSID2 tree.\n";
    return;
  }

  file.read(buf, 0x80);

  // add a comment line
  *out << "; second=" << rec.second << " listen=" << rec.number << endl;
  //     << " score=" << (int)rec.score << endl;

  *out << "file=" << rec.path << rec.name << endl;
  *out << "name=" << &buf[0x16] << endl;
  *out << "author=" << &buf[0x36] << endl;
  *out << "copyright=" << &buf[0x56] << endl;
  *out << "songs=" << (buf[0x0E]*256+buf[0x0F]) << endl;
  *out << "fadeout=0\n";
  *out << "subtune=" << rec.tune << endl << endl;

  file.close();
}

/**
 * Generate the playlist
 *
 *@param name the name of the output file
 */
void generatePlaylist(char *name) {
  TSIDExaminer2::TRec rec;

  // open the output file
  ofstream out(name, ios::out);

  if (!out) {
    cerr << "Error creating playlist file: " << name << endl;
    return;
  }

  // give standard playlist header
  out << "[SIDPLAY/Windows Playlist]\n";
  out << "Format=2\n";
  out << "Basedir=" << env << endl << endl;

  // init the engine for getting the songs
  examiner->initForGet();

  while (examiner->getNext(&rec)) {
    addPlaylistEntry(rec, &out);
  }

  // close the output file
  out.close();
}

/**
 * Show a help screen
 */
void showHelp() {
  cout << "Examiner2 1.00/";
  cout << TSIDExaminer2::getTSID2Version() << endl  <<endl;
  cout << "Syntax: examiner2 -<option>... [-<option>...]\n";
  cout << "Options:\n";
  cout << " -h          display this screen\n";
  cout << " -b<path>    set path for the base TSID2 directory\n";
  cout << " -x          do not use the index (less accurate engine)\n";   
  ///cout << " -i<path>    set path to the index.ini file\n";
  cout << "----\n";
  cout << " -u          show unsorted list\n";
  cout << " -s          show sorted list\n";
  cout << "----\n";
  cout << " -t          show single tune list (unsorted and/or sorted)\n";
  cout << " -f          show songs (files) list (unsorted and/or sorted)\n";
  cout << " -a          show author list (unsorted and/or sorted)\n";
  cout << "----\n";
  cout << " -l          show order list as listen time\n";
  cout << " -n          show order list as number of listen\n";
  //cout << " -p          show order list as pondered\n";
  cout << "----\n";
  cout << " -g          generate playlist files\n\n";
  cout << " default active options for each group: -s, -tfa, -l\n";
#ifdef HAVE_MSWINDOWS
  //cout << " pondered option is not jet active in windows\n";
#endif
  cout << endl;
}

int main(int argc, char** argv) {
  int result;                                // parameters results
  
  path=NULL;                                 // assume no path 
  home=NULL;                                 // assume no home
  ///ipath=NULL;                                // assume no path for index file

  // get parameters value
  result=getParameter(argc, argv);

  if (result & P_ERROR) {                    // there's a error?
    cout << "Invalid parameter option. Try 'examiner2 -h' for a syntax help\n";
    return 1;
  }

  if (result & P_HELP) {                     // help option?
    showHelp();
    return 0;
  }
  
  if (path==NULL) {
    //use default path
    if (!getHomeDir()) {
      cout << "I'm not able to found your home directory\n";
      return 8;
    }
    path=new char[strlen(home)+strlen(TSID2DIR)+2];
    if (path==NULL) {    
      // no memory error
      cerr << "No more memory\n";
      return 9;
    }
    sprintf(path, "%s/%s", home, TSID2DIR);
    convertDirSeperator(path);        
  }
  
  ///if (ipath==NULL) {
    //use default path for index
    
    /// to insert
  ///}
  
  if (result & P_NOINDEX)  examiner=new TSIDExaminer2(path, false);
  else examiner=new TSIDExaminer2(path, true);
  
  // look if examiner set up correctly
  if (!*examiner) {
    cerr << examiner->getError() << endl;
    if (examiner->getCodeError() == TSIDExaminer2::INI_OPEN_ERROR) {
      cerr << "Test if you have index.ini into your TSID2 directory, ";
      cerr << "or skip the use of index with -x option\n";
      ///cerr << "or specify it's location with the -i option\n";
    }
    return 7;
  }
  
  if ((result & P_PLAYLIST) || (result & P_PONDERED)) {
    //TSID tsid;
    //if (!tsid) {
    //  cerr << tsid.getError() << endl;
    //  return 6;
   // }
    //if (!tsid.setBaseDir()) {
    //  cerr << "Error: you not have a valid HVSC_DIR set up.\n";
    //  return 2;
    //}
    if (result & P_PONDERED) {
//#ifdef HAVE_UNIX
      // initialize the songlength database
/*      char *tmpName;

      tmpName=new char[strlen(env)+strlen(DOC)+strlen(DBN)+3];
      if (tmpName==NULL) {
        cerr << "Error allocating memory\n";
        return 3;
      }
      sprintf(tmpName, "%s%c%s%c%s", env, PATH_SEP, DOC, PATH_SEP, DBN);

      if (!examiner.openDB(tmpName)) {
        cerr << "Error opening the songlength database from DOCUMENTS directory\n";
        return 4;
      }
      examiner.setPath(env);
     examiner.activateLength(); */ 
//#else
      cerr << "Pondered option not yet supported\n";
      return 5;
//#endif
    }
  }

  // look for activating the default options: -s, -tfa, -l

  if (!(result & (P_UNSORTED | P_SORTED))) {
    result |= P_SORTED;                                 // assume default
  }

  if (!(result & (P_AUTHOR | P_SONG | P_TUNE))) {
    result |= (P_AUTHOR | P_SONG | P_TUNE);             // assume default
  }

  if (!(result & (P_TIME | P_NUMBER | P_PONDERED))) {
    result |= P_TIME;                                   // assume default
  }

  // analise the TSID2 directory
  examiner->analise();
  if (examiner->getCodeError()!=0) {
    cerr << examiner->getError() << endl;
  }

  if (result & P_TUNE) {
    if (result & P_UNSORTED) {
      if (result & P_PLAYLIST) {
        generatePlaylist(ULOST);
      }

      cout << "******** UNSORTED LIST OF SINGLE TUNES *********************\n";
      examiner->showList(&cout);
      cout << endl;
    }

    if ((result & P_SORTED) && (result & P_TIME)) {
      examiner->sort(TSIDExaminer2::TIME);

      if (result & P_PLAYLIST) {
        generatePlaylist(LTSLOST);
      }

      cout << "******** LISTEN TIME SORTED LIST OF SINGLE TUNES ***********\n";
      examiner->showList(&cout);
      cout << endl;
    }

    if ((result & P_SORTED) && (result & P_NUMBER)) {
      examiner->sort(TSIDExaminer2::NUMBER);

      if (result & P_PLAYLIST) {
        generatePlaylist(NLSLOST);
      }

      cout << "******** NUMBER LISTEN SORTED LIST OF SINGLE TUNES *********\n";
      examiner->showList(&cout);
      cout << endl;
    }

    if ((result & P_SORTED) && (result & P_PONDERED)) {
      examiner->sort(TSIDExaminer2::SCORE);
      if (result & P_PLAYLIST) {
        generatePlaylist(PSLOST);
      }
      cout << "******** PONDERED SORTED LIST OF SINGLE TUNES *************\n";
      examiner->showList(&cout, true);
      cout << endl;
    }
  }

  examiner->mergeSongList();
  if (result & P_SONG) {

    if (result & P_UNSORTED) {
      if (result & P_PLAYLIST) {
        generatePlaylist(ULOS);
      }

      cout << "******** UNSORTED LIST OF SONGS (FILES) ********************\n";
      examiner->showList(&cout);
      cout << endl;
    }

    if ((result & P_SORTED) && (result & P_TIME)) {
      examiner->sort(TSIDExaminer2::TIME);

      if (result & P_PLAYLIST) {
        generatePlaylist(LTSLOS);
      }

      cout << "******** LISTEN TIME SORTED LIST OF SONGS (FILES) **********\n";
      examiner->showList(&cout);
      cout << endl;
    }

    if ((result & P_SORTED) && (result & P_NUMBER)) {
      examiner->sort(TSIDExaminer2::NUMBER);

      if (result & P_PLAYLIST) {
        generatePlaylist(NLSLOS);
      }

      cout << "******** NUMBER LISTEN SORTED LIST OF SONGS (FILES) ********\n";
      examiner->showList(&cout);
      cout << endl;
    }

    if ((result & P_SORTED) && (result & P_PONDERED)) {
      examiner->sort(TSIDExaminer2::SCORE);
      if (result & P_PLAYLIST) {
        generatePlaylist(PSLOS);
      }
      cout << "******** PONDERED SORTED LIST OF SONGS (FILES) ************\n";
      examiner->showList(&cout, true);
      cout << endl;
    }
  }

  examiner->mergeAuthorList();
  if (result & P_AUTHOR) {

    if (result & P_UNSORTED) {
      cout << "******** UNSORTED LIST OF AUTHORS **************************\n";
      examiner->showList(&cout);
      cout << endl;
    }

    if ((result & P_SORTED) && (result & P_TIME)) {
      examiner->sort(TSIDExaminer2::TIME);
      cout << "******** LISTEN TIME SORTED LIST OF AUTHORS ****************\n";
      examiner->showList(&cout);
      cout << endl;
    }

    if ((result & P_SORTED) && (result & P_NUMBER)) {
      examiner->sort(TSIDExaminer2::NUMBER);
      cout << "******** NUMBER LISTEN SORTED LIST OF AUTHORS **************\n";
      examiner->showList(&cout);
      cout << endl;
    }

    if ((result & P_SORTED) && (result & P_PONDERED)) {
      examiner->sort(TSIDExaminer2::SCORE);
      cout << "******** PONDERED SORTED LIST OF AUTHORS ******************\n";
      examiner->showList(&cout, true);
      cout << endl;
    }
  }

  if (examiner->getCodeError()!=0) {
    cerr << examiner->getError() << endl;
  }
  
  delete examiner;
  delete path;
  if (home!=NULL) delete home;
  ///delete ipath;
  return 0;
}

// _Examiner2.CPP
