eval 'exec perl -w -S $0 ${1+"$@"}'
                if 0;

# Make the index of files for TSID2 examiner features
# 
# This program needs this information for making the file:
# 1) HVSC directory
# 2) CGSC directory

use Getopt::Std;
use Audio::SID 3.03;

##############################################################################
#
# GLOBAL VARIABLES
#
##############################################################################

my $VERSION = "1.00";

# Stuff for SID object.
my $mySID = new Audio::SID();
my (@SIDfields) = $mySID->getFieldNames();

my $HVSC_DIR;      # HVSC base directory
my $CGSC_DIR;      # CGSC base directory


# print the usage of this program
sub printUsage {
    print <<EOF;

USAGE: makeindex.pl  -s <HVSC base dir> 
                     -c <CGSC base dir>                     
                     [-h] [-?]

Makes the index file for TSID2 examiner features by scanning HVSC and CGSC directories

-s <HVSC base dir>: the HVSC path
-c <CGSC base dir>: the CGSC path
-h -?:              show an help screen            

EOF
}

# generate an output line of the  INI file
# @param path the path of the file
# @param name only the name of file
# @param base the base path of collection
sub generateLine {
  my ($path, $name, $base) = @_;

  if ($mySID->read('-filename' => $path.'/'.$name)) {
    $MD5=$mySID->getMD5;
  }
  
  my $rfile = $path;
  $rfile =~ s/$base//;

  print FH $MD5.'= '.$rfile.' '.$name."\n";
}

# recurse into directory
# the passed value is refer to hvsc or cgsc directory
# @param directory: the directory to recurse
# @param base the base path
sub recurse {
    my ($directory, $base)=@_;
    print 'Recurse into directory '.$directory.' with '.$base."\n";
    opendir(DIR, $directory);
    my @allFiles = sort {uc($a) cmp uc($b)} readdir(DIR);
    closedir(DIR);

    foreach $file (@allFiles) {

      next if ($file eq '.');
      next if ($file eq '..');
      if (-d $directory.'/'.$file) {
        recurse($directory.'/'.$file, $base);
        next;
      } 
      if (($file =~/\.sid/) or ($file =~/\.mus/)) {
        print $directory.'/'.$file."\n";
        generateLine($directory, $file, $base);
      }

      next;
    }

    return 1;
}


##############################################################################
#
# Main program starts here
#
##############################################################################


getopts('s:c:h?', \%opts);

if ($opts{h} or $opts{'?'}) {
    printUsage();
    exit;
}

if ($opts{s}) {
    $HVSC_DIR = $opts{s};
}

if ($opts{c}) {
    $CGSC_DIR = $opts{c};
}

if ($HVSC_DIR) {
    $HVSC_DIR =~ s/\s*$//;
    $HVSC_DIR =~ s/[\/\\]+$//; # Remove trailing slash or backslash, if any.
    print "HVSC Base dir: '$HVSC_DIR'\n";
}  else {
      print "No HVSC set. Exit!\n";
      exit;
   }

if ($CGSC_DIR) {
    $CGSC_DIR =~ s/\s*$//;
    $CGSC_DIR =~ s/[\/\\]+$//; # Remove trailing slash or backslash, if any.
    print "CGSC Base dir: '$CGSC_DIR'\n";
}  else {
      print "No New CGSC set. Exit!\n";
      exit;
   }   
   

open(FH, "> ./index.ini") || die $!;  #"

print FH "[TUNES]\n";
recurse($HVSC_DIR, $HVSC_DIR);
recurse($CGSC_DIR, $CGSC_DIR);

close (FH);
                
  
