eval 'exec perl -w -S $0 ${1+"$@"}'
                if 0;

# Make update TSID2  file
# 
# This program needs this information for making the TSID2 update file:
# 1) HVSC xx-1 directory
# 2) HVSC xx directory
# 3) UpdatePaths.pl output file for HVSC xx-1 to xx
#              
# UpdatePaths.pl
#   in 2000 by LaLa <LaLa@C64.org>


use Getopt::Std;
use Audio::SID 3.01;

##############################################################################
#
# GLOBAL VARIABLES
#
##############################################################################

my $VERSION = "1.00";

# Stuff for SID object.
my $mySID = new Audio::SID();
my (@SIDfields) = $mySID->getFieldNames();

my $HVSC_OLD_BASE;    # old HVSC base directory
my $HVSC_NEW_BASE;    # new HVSC base directory
my $UPDATE_FILE;      # update file


# print the usage of this program
sub printUsage {
    print <<EOF;

USAGE: makeupdate.pl -o <HVSC xx-1 base dir> 
                     -n <HVSC xx base dir>
                     -u <UpdateXX.upd>
                     [-h] [-?]

Makes the base update file for TSID2 by comparing old and new HVSC and using
the UpdateXX file generated from updatepaths.pl

-o <HVSC xx-1 base dir>: the HVSC (before the update) path
-n <HVSC xx base dir>:   the HVSC (after the update) path
-u UpdateXX.upd:         the updatepaths.pl output file for this update
-h -?:                   show an help screen            

EOF
}


# generate an output line in the TDIS2 update format
# if the md5 are equal, we don't make the output
# @param from the name of the file from old HVSC
# @param to the name of the file from new HVSC
sub generateLine {
  my ($from, $to) = @_;
  
  # avoid to count if there was a move and then a set clock
  if (open (INN, '< '.$HVSC_NEW_BASE.'/'.$to."\n")) {
    close (INN);
  
    if ($mySID->read('-filename' => $HVSC_OLD_BASE.'/'.$from)) {
      $fromMD5=$mySID->getMD5;
    } 
  
    if ($mySID->read('-filename' => $HVSC_NEW_BASE.'/'.$to)) {
      $toMD5=$mySID->getMD5;
    }   
  
    if ($fromMD5 != $toMD5) {
      print FH '% '.$from.' '.$to."\n";
      print FH "MOVE $fromMD5 $toMD5\n";
    }  
  }  
}

# generate an output line in the TDIS2 update format
# if the md5 are equal, we don't make the output
# @param from the name of the file from HVSC
# @param to the name of the file from HVSC
sub generateLine2 {
  my ($from, $to) = @_;
  
  # avoid to count if the file was new
  if (open (INN, '< '.$from."\n")) {
    close (INN);
  
    if ($mySID->read('-filename' => $from)) {
      $fromMD5=$mySID->getMD5;
    } 
  
    if ($mySID->read('-filename' => $to)) {
      $toMD5=$mySID->getMD5;
    }   
  
    if ($fromMD5 != $toMD5) {
      print FH '% '.$from.' '.$to."\n";
      print FH "MOVE $fromMD5 $toMD5\n";
    }  
  }  
}

# recurse into directory
# the passed value is refer to hvsc directory
sub recurse {
    my ($directory)=$_[0];
    
    my $new=$HVSC_OLD_BASE.'/update/'.$directory;
    print 'Recurse into directory '.$new."\n";
    opendir(DIR, $new);
    my @allFiles = sort {uc($a) cmp uc($b)} readdir(DIR);
    closedir(DIR);

    foreach $file (@allFiles) {
    
      next if ($file eq '.');
      next if ($file eq '..');
      if (-d $new.$file) {
        recurse($directory.$file.'/');
        next;
      }

      
      if ($file =~'.sid') {
        print $direcory.$file."\n";
        generateLine2($HVSC_OLD_BASE.'/'.$directory.$file, $new.$file); 
      }
      #if ($mySID->read('-filename' => $directory.$file)) {
      #}
      
      
      next;
    }
    
    
    return 1;
}

##############################################################################
#
# Main program starts here
#
##############################################################################


getopts('o:n:u:h?', \%opts);

if ($opts{h} or $opts{'?'}) {
    printUsage();
    exit;
}

if ($opts{o}) {
    $HVSC_OLD_BASE = $opts{o};
}

if ($opts{n}) {
    $HVSC_NEW_BASE = $opts{n};
}

if ($opts{u}) {
    $UPDATE_FILE = $opts{u};
}

if ($HVSC_OLD_BASE) {
    $HVSC_OLD_BASE =~ s/\s*$//;
    $HVSC_OLD_BASE =~ s/[\/\\]+$//; # Remove trailing slash or backslash, if any.
    print "HVSC Old Base dir: '$HVSC_OLD_BASE'\n";
}  else {
      print "No Old HVSC set. Exit!\n";
      exit;
   }

if ($HVSC_NEW_BASE) {
    $HVSC_NEW_BASE =~ s/\s*$//;
    $HVSC_NEW_BASE =~ s/[\/\\]+$//; # Remove trailing slash or backslash, if any.
    print "HVSC New Base dir: '$HVSC_NEW_BASE'\n";
}  else {
      print "No New HVSC set. Exit!\n";
      exit;
   }   
   
if ($opts{u}) {

    open(FH, "> ./update.thvs2") || die $!;  #"
 
    # Reading the changes to HVSC pathnames from update file...

    print "Processing $opts{u}\n";
    print "--- $opts{u} ---\n" if ($opts{u});

    open (IN, "< $opts{u}\n") or die "ERROR: Couldn't open $opts{u}!\n";

    $firstline = <IN>;

    if ($firstline =~ /^Processing (\/update\/Update\d\d\.hvs)\s*$/) {
        print "Reading changes made in $1...\n";
    }
    else {
        die "ERROR: '$opts{u}' is an invalid updatepaths.pl output file!\n";
    }

    while (<IN>) {
        s/\s*$//;

        if (/^MOVE: /) {
            s/^MOVE: //;
            s/\'//g;
            ($from, $to) = split(/ => /, $_);         
            print "MOVE: '$from' => '$to'\n";
            generateLine($from, $to);
        }
        elsif (/^DELETE: /) {
            s/^DELETE: //;
            s/\'//g;
            print "DELETE: '$_'\n";
        }
        elsif (/^CONTENT CHANGE: /) {
            s/^CONTENT CHANGE: //;
            ($file, $what) = ($_ =~ /\'(\S+)\' \((.+)\)$/);
            print "CONTENT CHANGE: '$file' ($what)\n";
            if ($what =~ /REPLACED/) {
              if (not($what =~ /ALSO/)) {
                  generateLine($file, $file);
              }               
            } else {
                generateLine($file, $file);
              }
            
        }
    }

    close (IN);

    print "--- END PROCESSING UPDATE---\n";
    
    # now processing all files that are in NEW/update for any MD5 changes:
    
    #$directory='';
    #recurse($direcotry);

    close (FH);
                
} else {
     print "No Update file specified. Exit!\n";
  }   
  
  
